USE Passengers

SELECT
    ID,
    STUFF	((
			SELECT '; ' + t.Names
			FROM Passenger t
			WHERE t.ID = Passenger.id
			ORDER BY t.Names
			FOR XML PATH('')
			),1,2,'') as Names
FROM Passenger
GROUP BY ID
; 

/*  Results
ID       Names
AA12345	Newby, Michael; Newby, William
AA12346	Appleseed, Jane; Appleseed, Johnny
AA12348	Jordan, Michael
AA12349	Wick, Helen; Wick, John
*/


/*
This query returns the list of Passengers (or any data) in alphabetical order to be placed in single cell in Excel.
The STUFF function inserts a string into another string. It deletes a specified length of characters in the first 
string at the start position provided and then inserts the second string into the first string at the start position.
*/

SELECT 
	STUFF	((
			SELECT
			/*  Add a semi-colon and space before each value  */
			/*  semi-colon can be removed or replaced with any "delimiting" character; I normally use comma  */
			'; ' + [Names]
			FROM Passenger
			GROUP BY Names
			/*  FOR XML PATH allows ORDER BY (and TOP) in subquery  */
			ORDER BY Names
			FOR XML PATH('')
			/*  Remove the first 2 characters (", ") from the start position of 1  */
			), 1, 2, ''
			) As Names

/*  RESULTS:  
Appleseed, Jane; Appleseed, Johnny; Jordan, Michael; Newby, Michael; Newby, William; Wick, Helen; Wick, John
*/

/*
This query returns same list as above query.
The difference is query below uses COALESCE instead of STUFF, FOR XML PATH.
*/
DECLARE @passengers varchar(8000)
SET @passengers = NULL

SELECT @passengers = COALESCE(@passengers + '; ','') + Names
FROM Passenger
ORDER BY Names

SELECT @passengers As Names

/*  RESULTS:  
Appleseed, Jane; Appleseed, Johnny; Jordan, Michael; Newby, Michael; Newby, William; Wick, Helen; Wick, John
*/



EXEC usp_GetPassengerList
/*  RESULTS:
Appleseed, Jane; Appleseed, Johnny; Jordon, Michael; Newby, Michael; Newby, William; Wick, Helen; Wick, John
*/

EXEC usp_GetPassengersByID 
/*  RESULTS:
ID		Names
AA12345	Newby, Michael; Newby, William
AA12346	Appleseed, Jane; Appleseed, Johnny
AA12348	Jordan, Michael
AA12349	Wick, Helen; Wick, John
*/

EXEC usp_GetPassengersByNames
/*  RESULTS:
ID		Names
AA12346	Appleseed, Jane; Appleseed, Johnny
AA12348	Jordan, Michael
AA12345	Newby, Michael; Newby, William
AA12349	Wick, Helen; Wick, John
*/


/*  From LinkedIn post by Luca Biondi
https://sqlserverperformace.blogspot.com/2021/10/sql-server-concatenates-text-from.html
*/

DECLARE @Names VARCHAR(8000)
SELECT @Names = ISNULL(@Names,'') + '; ' + Names FROM Passenger
SELECT STUFF(@Names,1,2,'')

/*  RESULTS:
Newby, Michael; Newby, Wiliam; Appleseed, Johnny; Appleseed, Jane; Jordan, Michael; Wick, John; Wick, Helen
*/

DECLARE @Names VARCHAR(8000)
SELECT @Names = ISNULL(@Names,'') + ';' + Names FROM Passenger
SELECT STUFF(@Names,1,1,'')

/*  RESULTS:
Newby, Michael;Newby, Wiliam;Appleseed, Johnny;Appleseed, Jane;Jordan, Michael;Wick, John;Wick, Helen
*/
